/**
 ** Registro.java
 **
 ** Copyright (C) 1992, 1999, 2007, 2011 Mariano Alvarez Fernandez
 ** [e-mail: malfer at telefonica.net]
 **
 ** This file is part of the Parchis4A program.
 **
 ** Parchis4A is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** Parchis4A is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with Parchis4A.  If not, see <http://www.gnu.org/licenses/>.
 **
 **/

package com.fgrim.parchis4a;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;

public class Registro {

//	private static final String SAVEREG_VERSION_1 = "parchis4a_savereg_1";

	private ArrayList<RegPartida> vregp;
	private int totalPartidas;
	public boolean cambiosPendientes = false;
	
	public Registro() {
		vregp = new ArrayList<RegPartida>();
		totalPartidas = 0;
	}
	
	public int size() {
		return vregp.size();
	}
	
	public int getTotal(){
		return totalPartidas;
	}
	
	public RegPartida getRp(int pos) {
		return vregp.get(pos);
	}
	
	public RegPartida busca(DefPartida dp) {
		for (RegPartida rp : vregp) {
			if (rp.dp.match(dp)) return rp;
		}
		return null;
	}
	
	public boolean registra(EstadoPartida ep) {
		if (ep.num_jg < 2) return false;  // solo si 2 o mas jugadores
		RegPartida rp;
		rp = busca(ep);
		if (rp == null) {
			rp = new RegPartida();
			rp.set(ep);
			vregp.add(rp);
		} else {
			rp.acumula(ep);
		}
		totalPartidas++;
		return true;
	}
	
	public void up(int pos) {
		if (pos == 0) return;
		RegPartida rp = vregp.get(pos-1);
		vregp.set(pos-1,vregp.get(pos));
		vregp.set(pos,rp);
	}
	
	public void down(int pos) {
		if (pos == vregp.size()-1) return;
		RegPartida rp = vregp.get(pos);
		vregp.set(pos,vregp.get(pos+1));
		vregp.set(pos+1,rp);
	}
	
	public void delete(int pos) {
		RegPartida rp = vregp.remove(pos);
		totalPartidas -= rp.numpartidas;
	}
	
    public void save(ObjectOutputStream oout) throws IOException {
        oout.writeInt(vregp.size());
		for (RegPartida rp : vregp) {
            oout.writeObject(rp.toArray());
		}
    }

    public void load(ObjectInputStream oin) throws IOException, ClassNotFoundException {
    	int tamano = oin.readInt();
    	int a[];
    	RegPartida rp;
    	for (int i=0; i<tamano; i++) {
        	a = (int[])oin.readObject();
			rp = new RegPartida();
			rp.set(a);
			vregp.add(rp);
	    	totalPartidas += rp.numpartidas;
    	}
    }

    public String toString() {
        String s = "";
		for (RegPartida rp : vregp) {
			s += rp.toString();
		}
		return s;
	}

}
