/**
 ** RegistroAct.java
 **
 ** Copyright (C) 1992, 1999, 2007, 2011 Mariano Alvarez Fernandez
 ** [e-mail: malfer at telefonica.net]
 **
 ** This file is part of the Parchis4A program.
 **
 ** Parchis4A is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** Parchis4A is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with Parchis4A.  If not, see <http://www.gnu.org/licenses/>.
 **
 **/

package com.fgrim.parchis4a;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TableRow;
import android.widget.TextView;

public class RegistroAct extends ListActivity {

    public static final int DIALOG_REG_DELETE_ID = 0;
	private Resources resG;
	private int posDelete;
	private boolean changed = false;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        resG = getResources();
        setTitulo();

    	setListAdapter(new RegListAdapter(this));
    	getListView().setDividerHeight(2);
    	registerForContextMenu(getListView());
    }
 
    private void setTitulo() {
    	String s = resG.getString(R.string.reg_activity) + ", " +
	    Parchis4A.mRegistro.getTotal() + " "; 
	    s += (Parchis4A.mRegistro.getTotal() == 1) ? resG.getString(R.string.reg_partida) :
		    resG.getString(R.string.reg_partidas);
	    this.setTitle(s);
    }

    @Override
    protected void onListItemClick(ListView l, View v, int pos, long id) {    
       ((RegListAdapter)getListAdapter()).toggle(pos);
    }
    
    @Override
    public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
        super.onCreateContextMenu(menu, v, menuInfo);
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.reg_context_menu, menu);
        AdapterContextMenuInfo info = (AdapterContextMenuInfo) menuInfo;
        RegPartida rp = Parchis4A.mRegistro.getRp(info.position);
        Bitmap bmst = BMPlayerGroup.Generate(resG, rp.dp, false);
        BitmapDrawable bmd = new BitmapDrawable(resG, bmst);
        menu.setHeaderIcon(bmd);
        menu.setHeaderTitle(" ");
    }

    @Override
    public boolean onContextItemSelected(MenuItem item) {
        AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
        switch (item.getItemId()) {
        case R.id.regc_menu_up:
        	Parchis4A.mRegistro.up(info.position);
        	Parchis4A.mRegistro.cambiosPendientes = true;
        	((RegListAdapter)getListAdapter()).notifyDataSetChanged();
            return true;
        case R.id.regc_menu_down:
        	Parchis4A.mRegistro.down(info.position);
        	Parchis4A.mRegistro.cambiosPendientes = true;
        	((RegListAdapter)getListAdapter()).notifyDataSetChanged();
            return true;
        case R.id.regc_menu_delete:
        	posDelete = info.position;
            showDialog(DIALOG_REG_DELETE_ID);
            return true;
        default:
            return super.onContextItemSelected(item);
        }
    }
    
    @Override
    protected Dialog onCreateDialog(int id) {
        Dialog dialog;

        switch(id) {
        case DIALOG_REG_DELETE_ID:
        	AlertDialog.Builder builderde = new AlertDialog.Builder(this);
        	builderde.setTitle(R.string.dlg_reg_delete_title);
        	builderde.setMessage(resG.getString(R.string.dlg_reg_delete_message));
        	builderde.setCancelable(false);
        	builderde.setPositiveButton(resG.getString(R.string.dlg_yes),
        	    new DialogInterface.OnClickListener() {
        	        public void onClick(DialogInterface dialog, int id) {
        	        	Parchis4A.mRegistro.delete(posDelete);
        	        	Parchis4A.mRegistro.cambiosPendientes = true;
        	        	((RegListAdapter)getListAdapter()).notifyDataSetChanged();
        	        	setTitulo();
        	        	dialog.cancel();
        	        }
        	    });
        	builderde.setNegativeButton(resG.getString(R.string.dlg_no),
        	    new DialogInterface.OnClickListener() {
        	        public void onClick(DialogInterface dialog, int id) {
   	                    dialog.cancel();
        	        }
        	    });
        	dialog = builderde.create();
        	break;
        default:
            dialog = null;
        }
        return dialog;
    }

    @Override
    protected void onPrepareDialog(int id, Dialog dialog) {
        switch(id) {
        case DIALOG_REG_DELETE_ID:
            RegPartida rp = Parchis4A.mRegistro.getRp(posDelete);
            Bitmap bmst = BMPlayerGroup.Generate(resG, rp.dp, false);
            BitmapDrawable bmd = new BitmapDrawable(resG, bmst);
            AlertDialog ad = (AlertDialog)dialog;
            ad.setIcon(bmd);
            break;
        }
    }

    private class RegListAdapter extends BaseAdapter {

        private LayoutInflater mInflater;
        private Context mContext;

    	public RegListAdapter(Context context) {
            mContext = context;
            mInflater = LayoutInflater.from(mContext);
        }

        public int getCount() {
            return Parchis4A.mRegistro.size();
        }

        public Object getItem(int pos) {
            return pos;
        }

        public long getItemId(int pos) {
            return pos;
        }

        public View getView(int pos, View convertView, ViewGroup parent) {
            ViewHolder holder;
            
            if (convertView == null) {
                convertView = mInflater.inflate(R.layout.regp_layout, null);
                holder = new ViewHolder();
                holder.icon = (ImageView) convertView.findViewById(R.id.regp_icon);
                holder.np_text = (TextView) convertView.findViewById(R.id.regp_np_text);
                holder.exp_ll = (LinearLayout) convertView.findViewById(R.id.regp_exp_ll);
                holder.tr[0] = (TableRow) convertView.findViewById(R.id.regp_r1);
                holder.tr[1] = (TableRow) convertView.findViewById(R.id.regp_r2);
                holder.tr[2] = (TableRow) convertView.findViewById(R.id.regp_r3);
                holder.tr[3] = (TableRow) convertView.findViewById(R.id.regp_r4);
                holder.lr[0][0] = (TextView) convertView.findViewById(R.id.regp_l11);
                holder.lr[0][1] = (TextView) convertView.findViewById(R.id.regp_l12);
                holder.lr[0][2] = (TextView) convertView.findViewById(R.id.regp_l13);
                holder.lr[0][3] = (TextView) convertView.findViewById(R.id.regp_l14);
                holder.lr[1][0] = (TextView) convertView.findViewById(R.id.regp_l21);
                holder.lr[1][1] = (TextView) convertView.findViewById(R.id.regp_l22);
                holder.lr[1][2] = (TextView) convertView.findViewById(R.id.regp_l23);
                holder.lr[1][3] = (TextView) convertView.findViewById(R.id.regp_l24);
                holder.lr[2][0] = (TextView) convertView.findViewById(R.id.regp_l31);
                holder.lr[2][1] = (TextView) convertView.findViewById(R.id.regp_l32);
                holder.lr[2][2] = (TextView) convertView.findViewById(R.id.regp_l33);
                holder.lr[2][3] = (TextView) convertView.findViewById(R.id.regp_l34);
                holder.lr[3][0] = (TextView) convertView.findViewById(R.id.regp_l41);
                holder.lr[3][1] = (TextView) convertView.findViewById(R.id.regp_l42);
                holder.lr[3][2] = (TextView) convertView.findViewById(R.id.regp_l43);
                holder.lr[3][3] = (TextView) convertView.findViewById(R.id.regp_l44);
                convertView.setTag(holder);
            } else {
                holder = (ViewHolder) convertView.getTag();
            }
            RegPartida rp = Parchis4A.mRegistro.getRp(pos);
        	Bitmap bmst = BMPlayerGroup.Generate(resG, rp.dp, false);
        	holder.icon.setImageBitmap(bmst);
        	String s = rp.numpartidas + " ";
        	s += (rp.numpartidas == 1) ? resG.getString(R.string.reg_partida) :
        		resG.getString(R.string.reg_partidas);
            holder.np_text.setText(s);
            holder.exp_ll.setVisibility(rp.expandir ? View.VISIBLE : View.GONE);
            for (int c=0; c<4; c++) {
            	if (rp.dp.tipo_jugador[c] == DefPartida.NOJUEGA) {
            		holder.tr[c].setVisibility(View.GONE);
            	} else {
            		holder.tr[c].setVisibility(View.VISIBLE);
            		for (int j=0; j<4; j++)
            		    holder.lr[c][j].setText(rp.ranking(c, j));
            	}
            }
            return convertView;
        }

        public void toggle(int pos) {
            RegPartida rp = Parchis4A.mRegistro.getRp(pos);
            rp.expandir = !rp.expandir;
            notifyDataSetChanged();
        }
    }

    static class ViewHolder {
        ImageView icon;
        TextView np_text;
        LinearLayout exp_ll;
        TableRow tr[] = new TableRow[4];
        TextView lr[][] = new TextView[4][4];
    }
}